--======================================
-- lua module sidereal_time
-- hour angle of vernal equinox 
-- = ra of meridian
-- = ra of clock position as seen from earth's center

--======================================
-- ra,dec calculations
--======================================
-- the original xyz to RA,Dec routines were taken from Lua Edu's celutil.lua
-- Thanks, Vincent!

--- all other code  Selden Ball, June , 2008. All rights reserved. 

-- useful constants:

pkg = {
	LOOK  = celestia:newvector(0,0,-1);
	XAxis = celestia:newvector(1,0,0);
	YAxis = celestia:newvector(0,1,0);
	ZAxis = celestia:newvector(0,0,1);
	Second = 1.0/(24.0*60*60);
}
	pkg.yPi = celestia:newrotation( pkg.YAxis, math.pi)

-- Transform coordinates from cartesian to polar 
pkg.transform_xyz2rtp = function (x,y,z)
  local r = math.sqrt(x*x + y*y + z*z)
  local theta = math.atan(math.sqrt(x*x + y*y)/z)
  local phi   = math.atan2(y,x)
  return r,theta,phi
end


-- Return current Ra, Dec for selection
pkg.get_ra_dec = function (vp, sel)

  local base_rot = celestia:newrotation(celestia:newvector(1,0,0), -math.rad(23.4392911))
  local rot = vp:getposition():orientationto(sel:getposition(), pkg.LOOK) * base_rot

  local look = rot:transform(pkg.LOOK):normalize()
  local r,theta,phi = pkg.transform_xyz2rtp(look.x, look.z, look.y)


  local phi = math.mod(720 - math.deg(phi), 360)

  local theta = math.deg(theta)
  if theta > 0 then
    theta = 90 - theta
  else
    theta = (-90 - theta)
  end

  return phi, theta
end

--======================================
-- clock's RA as seen by Earth = local sidereal time
--======================================

SID_hour = function (date)
	local sel = celestia:find("Sol/Earth/Sidereal_Clock")
	local vp = celestia:find("Sol/Earth")
	local RA,Decl  = pkg.get_ra_dec (vp,sel)
	return RA/15.0
end

--==================================================================
SID_HoursState = function(date)
	local Angle = -(2.0*SID_hour(date)*(2*math.pi/24.0))
 	return  pkg.yPi*celestia:newrotation( pkg.ZAxis, Angle)
end


--==================================================================
SID_Hoursproto =
{
   Period    = 1,
}

-- constructor method
function SID_Hoursproto:new(o)
   o = o or {}  -- create table if one not provided
   setmetatable(o, self)
   self.__index = self

   -- set the period to whatever value was specified in the ssc file;
   -- slightly confusing because Celestia is case sensitive--period must
   -- be lowercase, but the field from the ssc file is capitalized.
   o.period = o.Period

   return o
end

-- The orientation function.

function SID_Hoursproto:orientation(tjd)

	local qNow = 	SID_HoursState(tjd)
	return qNow.w, qNow.x, qNow.y, qNow.z

end

function SID_Hours(sscvals)
   -- create a new SID_Hours rotation object
   return SID_Hoursproto:new(sscvals)
end

--==================================================================
--==================================================================
-- Sidereal Time_Minutes functions
--==================================================================

SID_MinutesState = function(date)

	local Angle = -SID_hour(date)* 2.0*math.pi

 	return  pkg.yPi*celestia:newrotation( pkg.ZAxis, Angle)
end


--==================================================================
SID_Minutesproto =
{
   Period    = 1,
}

-- constructor method
function SID_Minutesproto:new(o)
   o = o or {}  -- create table if one not provided
   setmetatable(o, self)
   self.__index = self

   -- set the period to whatever value was specified in the ssc file;
   -- slightly confusing because Celestia is case sensitive--period must
   -- be lowercase, but the field from the ssc file is capitalized.
   o.period = o.Period

   return o
end

-- The orientation function.

function SID_Minutesproto:orientation(tjd)

	local qNow = 	SID_MinutesState(tjd)
	return qNow.w, qNow.x, qNow.y, qNow.z

end

function SID_Minutes(sscvals)
   -- create a new SID_Minutes rotation object
   return SID_Minutesproto:new(sscvals)
end
--==================================================================
--==================================================================
-- Sidereal Time_Seconds functions
--==================================================================

SID_SecondsState = function(date)
	local Angle = -SID_hour(date)* 2.0*math.pi*60.0
 	return  pkg.yPi*celestia:newrotation( pkg.ZAxis, Angle)
end


--==================================================================
SID_Secondsproto =
{
   Period    = 1,
}

-- constructor method
function SID_Secondsproto:new(o)
   o = o or {}  -- create table if one not provided
   setmetatable(o, self)
   self.__index = self

   -- set the period to whatever value was specified in the ssc file;
   -- slightly confusing because Celestia is case sensitive--period must
   -- be lowercase, but the field from the ssc file is capitalized.
   o.period = o.Period

   return o
end

-- The orientation function.

function SID_Secondsproto:orientation(tjd)

	local qNow = 	SID_SecondsState(tjd)
	return qNow.w, qNow.x, qNow.y, qNow.z

end

function SID_Seconds(sscvals)
   -- create a new SID_Seconds rotation object
   return SID_Secondsproto:new(sscvals)
end
